'use strict';

var flat = require('flat');

function same(collect, path, args) {
  if (
    args.baseHasProperty
    && args.targetHasProperty
    && args.flatBase[path] === args.flatTarget[path]
  ) {
    collect.push({
      path,
      type: 'same',
      value: args.flatTarget[path]
    });

    return true;
  }

  return false;
}

function put(collect, path, args) {
  if (args.baseHasProperty && args.targetHasProperty) {
    collect.push({
      path,
      prev: args.flatBase[path],
      type: 'put',
      value: args.flatTarget[path]
    });

    return true;
  }

  return false;
}

function del(collect, path, args) {
  if (args.baseHasProperty && !args.targetHasProperty) {
    collect.push({
      path,
      type: 'del',
      value: args.flatBase[path]
    });

    return true;
  }

  return false;
}

function post(collect, path, args) {

  if (!args.baseHasProperty && args.targetHasProperty) {
    collect.push({
      path,
      type: 'post',
      value: args.flatTarget[path]
    });

    return true;
  }

  return false;
}

var checks = {
  same,
  put,
  del,
  post
};

function reconcile(mergeKeys, flatBase, flatTarget) {
  return mergeKeys.reduce((collect, path) => {

    const args = {
      flatBase,
      flatTarget,
      baseHasProperty: flatBase.hasOwnProperty(path),
      targetHasProperty: flatTarget.hasOwnProperty(path)
    };

    [
      checks.same,
      checks.put,
      checks.del,
      checks.post
    ].some((fn) => fn(collect, path, args));

    return collect;
  }, []);
}

function diff(base, target, opts) {
  const flatBase = flat.flatten(base, opts || {});
  const flatTarget = flat.flatten(target, opts || {});

  const mergeKeys = Object.keys(Object.assign({}, flatBase, flatTarget)).sort();

  return reconcile(mergeKeys, flatBase, flatTarget);
}

module.exports = diff;
